@@ -30,10 +30,11 @@ using System.IO;
 using System.Reflection;
 using System.Runtime.InteropServices;
 using System.Threading;
-
-using Gtk;
+using Thread = System.Threading.Thread;
+using GLib;
 
 using Beagle.Util;
+using Log = Beagle.Util.Log;
 
 #if ENABLE_WEBSERVICES
 using Beagle.WebService;
@@ -43,6 +44,7 @@ namespace Beagle.Daemon {
 	class BeagleDaemon {
 
 		public static Thread MainLoopThread = null;
+		private static MainLoop main_loop = null;
 
 		private static Server server = null;
 
@@ -53,6 +55,7 @@ namespace Beagle.Daemon {
 		public static bool StartServer ()
 		{
 			Logger.Log.Debug ("Starting messaging server");
+
 			try {
 				server = new Server ("socket");
 				server.Start ();
@@ -135,12 +138,12 @@ namespace Beagle.Daemon {
 
 		public static bool StartupProcess ()
 		{
+			Log.Debug ("Beginning main loop");
+
 			// Profile our initialization
 			Stopwatch stopwatch = new Stopwatch ();
 			stopwatch.Start ();
 
-			SetupSignalHandlers ();
-
 			// Fire up our server
 			if (! StartServer ()) {
 				if (arg_replace)
@@ -172,12 +175,24 @@ namespace Beagle.Daemon {
 			Logger.Log.Debug ("Starting QueryDriver");
 			QueryDriver.Start ();
 
+			bool initially_on_battery = SystemInformation.UsingBattery && ! Conf.Indexing.IndexOnBattery;
+
 			// Start the Global Scheduler thread
 			if (! arg_disable_scheduler) {
-				Logger.Log.Debug ("Starting Scheduler thread");
-				Scheduler.Global.Start ();
+				if (! initially_on_battery) {
+					Logger.Log.Debug ("Starting Scheduler thread");
+					Scheduler.Global.Start ();
+				} else
+					Log.Debug ("Beagle started on battery, not starting scheduler thread");
 			}
 
+			// Poll the battery status so we can shut down the
+			// scheduler if needed.  Ideally at some point this
+			// will become some sort of D-BUS signal, probably from
+			// something like gnome-power-manager.
+			prev_on_battery = initially_on_battery;
+			GLib.Timeout.Add (5000, CheckBatteryStatus);
+
 			// Start our Inotify threads
 			Inotify.Start ();
 	
@@ -189,8 +204,6 @@ namespace Beagle.Daemon {
 			//Beagle Web, WebService access initialization code:
 			WebServiceBackEnd.Start();
 #endif
-			Shutdown.ShutdownEvent += OnShutdown;
-
 			Conf.WatchForUpdates ();
 
 			stopwatch.Stop ();
@@ -206,7 +219,7 @@ namespace Beagle.Daemon {
 			return false;
 		}
 
-		static void OnEmptySchedulerQueue ()
+		private static void OnEmptySchedulerQueue ()
 		{
 			Logger.Log.Debug ("Scheduler queue is empty: terminating immediately");
 			Shutdown.BeginShutdown ();
@@ -218,12 +231,14 @@ namespace Beagle.Daemon {
 			try {
 				DoMain (args);
 			} catch (Exception ex) {
-				Logger.Log.Error ("Unhandled exception thrown.  Exiting immediately.");
-				Logger.Log.Error (ex);
+				Logger.Log.Error (ex, "Unhandled exception thrown.  Exiting immediately.");
 				Environment.Exit (1);
 			}
 		}
 
+		[DllImport("libgobject-2.0.so")]
+		static extern void g_type_init ();
+
 		public static void DoMain (string[] args)
 		{
 			SystemInformation.SetProcessName ("beagled");
@@ -248,6 +263,7 @@ namespace Beagle.Daemon {
 					break;
 
 				case "--heap-buddy":
+				case "--mdb":
 					// Silently ignore the --heap-buddy argument: it gets handled
 					// in the wrapper script.
 					break;
@@ -445,13 +461,24 @@ namespace Beagle.Daemon {
 			// Do BEAGLE_EXERCISE_THE_DOG_HARDER-related processing.
 			ExerciseTheDogHarder ();
 
-			if (Application.InitCheck ("beagled", ref args))
+			// Initialize GObject type system
+			g_type_init ();
+			
+			if (SystemInformation.XssInit ())
 				Logger.Log.Debug ("Established a connection to the X server");
 			else
 				Logger.Log.Debug ("Unable to establish a connection to the X server");
-
 			XSetIOErrorHandler (BeagleXIOErrorHandler);
 
+			QueryDriver.Init ();
+			Server.Init ();
+
+			SetupSignalHandlers ();
+			Shutdown.ShutdownEvent += OnShutdown;
+
+			main_loop = new MainLoop ();
+			Shutdown.RegisterMainLoop (main_loop);
+
 			// Defer all actual startup until the main loop is
 			// running.  That way shutdowns during the startup
 			// process work correctly.
@@ -459,8 +486,7 @@ namespace Beagle.Daemon {
 
 			// Start our event loop.
 			Logger.Log.Debug ("Starting main loop");
-
-			Application.Run ();
+			main_loop.Run ();
 
 			// If we placed our sockets in a temp directory, try to clean it up
 			// Note: this may fail because the helper is still running
@@ -478,6 +504,27 @@ namespace Beagle.Daemon {
 			}
 		}
 
+		/////////////////////////////////////////////////////////////////////////////
+
+		private static bool prev_on_battery = false;
+
+		private static bool CheckBatteryStatus ()
+		{
+			if (prev_on_battery && (! SystemInformation.UsingBattery || Conf.Indexing.IndexOnBattery)) {
+				if (! SystemInformation.UsingBattery)
+					Log.Info ("Deletected a switch from battery to AC power.  Restarting scheduler.");
+				Scheduler.Global.Start ();
+				prev_on_battery = false;
+			} else if (! prev_on_battery && SystemInformation.UsingBattery && ! Conf.Indexing.IndexOnBattery) {
+				Log.Info ("Detected a switch from AC power to battery.  Stopping scheduler.");
+				Scheduler.Global.Stop ();
+				prev_on_battery = true;
+			}
+
+			return true;
+		}
+
+
 		/////////////////////////////////////////////////////////////////////////////
 
 		private delegate int XIOErrorHandler (IntPtr display);
@@ -502,7 +549,7 @@ namespace Beagle.Daemon {
 
 		/////////////////////////////////////////////////////////////////////////////
 
-		static void SetupSignalHandlers ()
+		private static void SetupSignalHandlers ()
 		{
 			// Force OurSignalHandler to be JITed
 			OurSignalHandler (-1);
@@ -510,52 +557,44 @@ namespace Beagle.Daemon {
 			// Set up our signal handler
 			Mono.Unix.Native.Stdlib.signal (Mono.Unix.Native.Signum.SIGINT, OurSignalHandler);
 			Mono.Unix.Native.Stdlib.signal (Mono.Unix.Native.Signum.SIGTERM, OurSignalHandler);
+			Mono.Unix.Native.Stdlib.signal (Mono.Unix.Native.Signum.SIGUSR1, OurSignalHandler);
 
 			// Ignore SIGPIPE
 			Mono.Unix.Native.Stdlib.signal (Mono.Unix.Native.Signum.SIGPIPE, Mono.Unix.Native.Stdlib.SIG_IGN);
 		}
 
-		// Our handler triggers an orderly shutdown when it receives a signal.
-		// However, this can be annoying if the process gets wedged during
-		// shutdown.  To deal with that case, we make a note of the time when
-		// the first signal comes in, and we allow signals to unconditionally
-		// kill the process after 5 seconds have passed.
-		static DateTime signal_time = DateTime.MinValue;
-		static void OurSignalHandler (int signal)
+		// Mono signal handler allows setting of global variables;
+		// anything else e.g. function calls, even reentrant native methods are risky
+		private static void OurSignalHandler (int signal)
 		{
 			// This allows us to call OurSignalHandler w/o doing anything.
 			// We want to call it once to ensure that it is pre-JITed.
 			if (signal < 0)
 				return;
 
+			// Set shutdown flag to true so that other threads can stop initializing
+			if ((Mono.Unix.Native.Signum) signal != Mono.Unix.Native.Signum.SIGUSR1)
+				Shutdown.ShutdownRequested = true;
+
+			// Do all signal handling work in the main loop and not in the signal handler.
+			GLib.Idle.Add (new GLib.IdleHandler (delegate () { HandleSignal (signal); return false; }));
+		}
+
+		private static void HandleSignal (int signal)
+		{
 			Logger.Log.Debug ("Handling signal {0} ({1})", signal, (Mono.Unix.Native.Signum) signal);
 
-			bool first_signal = false;
-			if (signal_time == DateTime.MinValue) {
-				signal_time = DateTime.Now;
-				first_signal = true;
+			// If we get SIGUSR1, turn the debugging level up.
+			if ((Mono.Unix.Native.Signum) signal == Mono.Unix.Native.Signum.SIGUSR1) {
+				LogLevel old_level = Log.Level;
+				Log.Level = LogLevel.Debug;
+				Log.Debug ("Moving from log level {0} to Debug", old_level);
+				GLib.Idle.Add (new GLib.IdleHandler (delegate () { RemoteIndexer.SignalRemoteIndexer (); return false; }));
+				return;
 			}
 
-			if (Shutdown.ShutdownRequested) {
-				
-				if (first_signal) {
-					Logger.Log.Debug ("Shutdown already in progress.");
-				} else {
-					double t = (DateTime.Now - signal_time).TotalSeconds;
-					const double min_t = 5;
-
-					if (t < min_t) {
-						Logger.Log.Debug ("Signals can force an immediate shutdown in {0:0.00}s", min_t-t);
-					} else {
-						Logger.Log.Debug ("Forcing immediate shutdown.");
-						Environment.Exit (0);
-					}
-				}
-
-			} else {
-				Logger.Log.Debug ("Initiating shutdown in response to signal.");
-				Shutdown.BeginShutdown ();
-			}
+			Logger.Log.Debug ("Initiating shutdown in response to signal.");
+			Shutdown.BeginShutdown ();
 		}
 
 		/////////////////////////////////////////////////////////////////////////////
@@ -568,11 +607,12 @@ namespace Beagle.Daemon {
 			// Stop our Inotify threads
 			Inotify.Stop ();
 
-			// Shut down the global scheduler
-			Scheduler.Global.Stop ();
+			// Stop the global scheduler and ask it to shutdown
+			Scheduler.Global.Stop (true);
 
 			// Stop the messaging server
-			server.Stop ();
+			if (server != null)
+				server.Stop ();
 		}
 
 		/////////////////////////////////////////////////////////////////////////////
@@ -632,4 +672,5 @@ namespace Beagle.Daemon {
 			}
 		}
 	}
+
 }
