@@ -91,13 +91,13 @@ namespace Beagle.Daemon {
 					command.CommandText =
 						"SELECT version, fingerprint FROM db_info";
 					try {
-						reader = ExecuteReaderOrWait (command);
+						reader = SqliteUtils.ExecuteReaderOrWait (command);
 					} catch (Exception ex) {
 						Logger.Log.Warn ("Likely sqlite database version mismatch trying to read from {0}.  Purging.", GetDbPath (directory));
 						create_new_db = true;
 					}
 					if (reader != null && ! create_new_db) {
-						if (ReadOrWait (reader)) {
+						if (SqliteUtils.ReadOrWait (reader)) {
 							stored_version = reader.GetInt32 (0);
 							stored_fingerprint = reader.GetString (1);
 						}
@@ -117,28 +117,32 @@ namespace Beagle.Daemon {
 				File.Delete (GetDbPath (directory));
 				connection = Open (directory);
 
-				DoNonQuery ("CREATE TABLE db_info (             " +
-					    "  version       INTEGER NOT NULL,  " +
-					    "  fingerprint   STRING NOT NULL    " +
-					    ")");
+				SqliteUtils.DoNonQuery (connection,
+							"CREATE TABLE db_info (             " +
+							"  version       INTEGER NOT NULL,  " +
+							"  fingerprint   STRING NOT NULL    " +
+							")");
 
-				DoNonQuery ("INSERT INTO db_info (version, fingerprint) VALUES ({0}, '{1}')",
-					    VERSION, index_fingerprint);
+				SqliteUtils.DoNonQuery (connection,
+							"INSERT INTO db_info (version, fingerprint) VALUES ({0}, '{1}')",
+							VERSION, index_fingerprint);
 
-				DoNonQuery ("CREATE TABLE file_attributes (           " +
-					    "  unique_id      STRING UNIQUE,          " +
-					    "  directory      STRING NOT NULL,        " +
-					    "  filename       STRING NOT NULL,        " +
-					    "  last_mtime     STRING NOT NULL,        " +
-					    "  last_attrtime  STRING NOT NULL,        " +
-					    "  filter_name    STRING NOT NULL,        " +
-					    "  filter_version STRING NOT NULL         " +
-					    ")");
+				SqliteUtils.DoNonQuery (connection,
+							"CREATE TABLE file_attributes (           " +
+							"  unique_id      STRING UNIQUE,          " +
+							"  directory      STRING NOT NULL,        " +
+							"  filename       STRING NOT NULL,        " +
+							"  last_mtime     STRING NOT NULL,        " +
+							"  last_attrtime  STRING NOT NULL,        " +
+							"  filter_name    STRING NOT NULL,        " +
+							"  filter_version STRING NOT NULL         " +
+							")");
 
-				DoNonQuery ("CREATE INDEX file_path on file_attributes (" +
-					    "  directory,      " +
-					    "  filename        " +
-					    ")");
+				SqliteUtils.DoNonQuery (connection,
+							"CREATE INDEX file_path on file_attributes (" +
+							"  directory,      " +
+							"  filename        " +
+							")");
 			} else {
 				SqliteCommand command;
 				SqliteDataReader reader;
@@ -151,9 +155,9 @@ namespace Beagle.Daemon {
 				command.Connection = connection;
 				command.CommandText = "SELECT directory, filename FROM file_attributes";
 
-				reader = ExecuteReaderOrWait (command);
+				reader = SqliteUtils.ExecuteReaderOrWait (command);
 
-				while (ReadOrWait (reader)) {
+				while (SqliteUtils.ReadOrWait (reader)) {
 
 					string dir = reader.GetString (0);
 					string file = reader.GetString (1);
@@ -189,61 +193,6 @@ namespace Beagle.Daemon {
 			return c;
 		}
 
-		private void DoNonQuery (string format, params object [] args)
-		{
-			SqliteCommand command;
-			command = new SqliteCommand ();
-			command.Connection = connection;
-			command.CommandText = String.Format (format, args);
-
-			while (true) {
-				try {
-					command.ExecuteNonQuery ();
-					break;
-				} catch (SqliteBusyException ex) {
-					Thread.Sleep (50);
-				}
-			}
-
-			command.Dispose ();
-		}
-
-		private SqliteCommand QueryCommand (string where_format, params object [] where_args)
-		{
-			SqliteCommand command;
-			command = new SqliteCommand ();
-			command.Connection = connection;
-			command.CommandText =
-				"SELECT unique_id, directory, filename, last_mtime, last_attrtime, filter_name, filter_version " +
-				"FROM file_attributes WHERE " + 
-				String.Format (where_format, where_args);
-			return command;
-		}
-
-		static private SqliteDataReader ExecuteReaderOrWait (SqliteCommand command)
-		{
-			SqliteDataReader reader = null;
-			while (reader == null) {
-				try {
-					reader = command.ExecuteReader ();
-				} catch (SqliteBusyException ex) {
-					Thread.Sleep (50);
-				}
-			}
-			return reader;
-		}
-
-		static private bool ReadOrWait (SqliteDataReader reader)
-		{
-			while (true) {
-				try {
-					return reader.Read ();
-				} catch (SqliteBusyException ex) {
-					Thread.Sleep (50);
-				}
-			}
-		}
-
 		private FileAttributes GetFromReader (SqliteDataReader reader)
 		{
 			FileAttributes attr = new FileAttributes ();
@@ -288,6 +237,10 @@ namespace Beagle.Daemon {
 
 		public FileAttributes Read (string path)
 		{
+			// Sanitize the path; remove the last '/'
+			if (path != null && path != "/" && path.EndsWith ("/"))
+				path = path.TrimEnd ('/');
+
 			SqliteCommand command;
 			SqliteDataReader reader;
 
@@ -302,14 +255,15 @@ namespace Beagle.Daemon {
 			string directory = FileSystem.GetDirectoryNameRootOk (path).Replace ("'", "''");
 			string filename = Path.GetFileName (path).Replace ("'", "''");
 			lock (connection) {
-				command = QueryCommand ("directory='{0}' AND filename='{1}'",
-							directory, filename);
-				reader = ExecuteReaderOrWait (command);
+				command = SqliteUtils.QueryCommand (connection,
+								    "directory='{0}' AND filename='{1}'",
+								    directory, filename);
+				reader = SqliteUtils.ExecuteReaderOrWait (command);
 				
-				if (ReadOrWait (reader)) {
+				if (SqliteUtils.ReadOrWait (reader)) {
 					attr = GetFromReader (reader);
 					
-					if (ReadOrWait (reader))
+					if (SqliteUtils.ReadOrWait (reader))
 						found_too_many = true;
 				}
 				reader.Close ();
@@ -321,8 +275,9 @@ namespace Beagle.Daemon {
 				// and which isn't, we delete them all and return
 				// null.  (Which in most cases will force a re-index.
 				if (found_too_many) {
-					DoNonQuery ("DELETE FROM file_attributes WHERE directory='{0}' AND filename='{1}'",
-						    directory, filename);
+					SqliteUtils.DoNonQuery (connection,
+								"DELETE FROM file_attributes WHERE directory='{0}' AND filename='{1}'",
+								directory, filename);
 				}
 			}
 
@@ -346,21 +301,26 @@ namespace Beagle.Daemon {
 					filter_name = "";
 				filter_name = filter_name.Replace ("'", "''");
 
-				DoNonQuery ("INSERT OR REPLACE INTO file_attributes " +
-					    " (unique_id, directory, filename, last_mtime, last_attrtime, filter_name, filter_version) " +
-					    " VALUES ('{0}', '{1}', '{2}', '{3}', '{4}', '{5}', '{6}')",
-					    GuidFu.ToShortString (fa.UniqueId),
-					    fa.Directory.Replace ("'", "''"), fa.Filename.Replace ("'", "''"),
-					    StringFu.DateTimeToString (fa.LastWriteTime),
-					    StringFu.DateTimeToString (fa.LastAttrTime),
-					    filter_name,
-					    fa.FilterVersion);
+				SqliteUtils.DoNonQuery (connection,
+							"INSERT OR REPLACE INTO file_attributes " +
+							" (unique_id, directory, filename, last_mtime, last_attrtime, filter_name, filter_version) " +
+							" VALUES ('{0}', '{1}', '{2}', '{3}', '{4}', '{5}', '{6}')",
+							GuidFu.ToShortString (fa.UniqueId),
+							fa.Directory.Replace ("'", "''"), fa.Filename.Replace ("'", "''"),
+							StringFu.DateTimeToString (fa.LastWriteTime),
+							StringFu.DateTimeToString (fa.LastAttrTime),
+							filter_name,
+							fa.FilterVersion);
 			}
 			return true;
 		}
 
 		public void Drop (string path)
 		{
+			// Sanitize the path; remove the last '/'
+			if (path != null && path != "/" && path.EndsWith ("/"))
+				path = path.TrimEnd ('/');
+
 			// We don't want to "UnSetPathFlag" here, since we have no way of knowing
 			// if another path hashes to the same value as this one.
 
@@ -373,15 +333,16 @@ namespace Beagle.Daemon {
 				// If a transaction has been requested, start it now.
 				MaybeStartTransaction ();
 
-				DoNonQuery ("DELETE FROM file_attributes WHERE directory='{0}' AND filename='{1}'",
-					    directory, filename);
+				SqliteUtils.DoNonQuery (connection,
+							"DELETE FROM file_attributes WHERE directory='{0}' AND filename='{1}'",
+							directory, filename);
 			}
 		}
 
 		private void MaybeStartTransaction ()
 		{
 			if (transaction_state == TransactionState.Requested) {
-				DoNonQuery ("BEGIN");
+				SqliteUtils.DoNonQuery (connection, "BEGIN");
 				transaction_state = TransactionState.Started;
 			}
 		}
@@ -396,7 +357,7 @@ namespace Beagle.Daemon {
 		{
 			if (transaction_state == TransactionState.Started) {
 				lock (connection)
-					DoNonQuery ("COMMIT");
+					SqliteUtils.DoNonQuery (connection, "COMMIT");
 			}
 			transaction_state = TransactionState.None;
 		}
@@ -406,7 +367,7 @@ namespace Beagle.Daemon {
 			lock (connection) {
 				if (transaction_count > 0) {
 					Logger.Log.Debug ("Flushing requested -- committing sqlite transaction");
-					DoNonQuery ("COMMIT");
+					SqliteUtils.DoNonQuery (connection, "COMMIT");
 					transaction_count = 0;
 				}
 			}
@@ -430,9 +391,9 @@ namespace Beagle.Daemon {
 					"SELECT unique_id, directory, filename, last_mtime, last_attrtime, filter_name, filter_version " +
 					"FROM file_attributes";
 				
-				reader = ExecuteReaderOrWait (command);
+				reader = SqliteUtils.ExecuteReaderOrWait (command);
 				
-				while (ReadOrWait (reader)) {
+				while (SqliteUtils.ReadOrWait (reader)) {
 					attributes.Add (GetFromReader (reader));
 				}
 				reader.Close ();
