@@ -37,34 +37,56 @@ namespace Beagle.IndexHelper {
 	public class RemoteIndexerExecutor : RequestMessageExecutor {
 
 		static public int Count = 0;
+		static Hashtable indexer_table = new Hashtable ();
 
-		Hashtable indexer_table = new Hashtable ();
+		Indexable[] child_indexables;
+		FilteredStatus[] uris_filtered;
 
 		public override ResponseMessage Execute (RequestMessage raw_request)
 		{
-			RemoteIndexerRequest request = (RemoteIndexerRequest) raw_request;
+			RemoteIndexerRequest remote_request = (RemoteIndexerRequest) raw_request;
+
+			IndexHelperTool.ReportActivity ();
 
 			// Find the appropriate driver for this request.
-			IIndexer indexer = indexer_table [request.RemoteIndexName] as IIndexer;
-			if (indexer == null) {
-				// FIXME: This shouldn't be hard-wired.
-				if (request.RemoteIndexName == "FileSystemIndex")
-					indexer = new RenamingLuceneDriver (request.RemoteIndexName,
-									    request.RemoteIndexMinorVersion);
-				else
-					indexer = new LuceneDriver (request.RemoteIndexName,
-								    request.RemoteIndexMinorVersion);
-				indexer_table [request.RemoteIndexName] = indexer;
+			IIndexer indexer;
+			lock (indexer_table) {
+				indexer = indexer_table [remote_request.RemoteIndexName] as IIndexer;
+				if (indexer == null) {
+					indexer = new LuceneIndexingDriver (remote_request.RemoteIndexName,
+									    remote_request.RemoteIndexMinorVersion);
+					indexer_table [remote_request.RemoteIndexName] = indexer;
+				}
 			}
 
-			request.Process (indexer);
+			IndexerReceipt [] receipts = null;
+			if (remote_request.Request != null) // If we just want the item count, this will be null
+				receipts = indexer.Flush (remote_request.Request);
 
-			// Construct a response containing the item count.
+			// Child indexables probably have streams
+			// associated with them.  We need to store them before
+			// sending them back to the daemon.
+			if (receipts != null) {
+				foreach (IndexerReceipt r in receipts) {
+					IndexerChildIndexablesReceipt cir;
+					cir = r as IndexerChildIndexablesReceipt;
+					if (cir != null) {
+						foreach (Indexable i in cir.Children)
+							i.StoreStream ();
+					}
+				}
+			}
+
+			// Construct a response containing the item count and
+			// the receipts produced by the actual indexing.
 			RemoteIndexerResponse response = new RemoteIndexerResponse ();
 			response.ItemCount = indexer.GetItemCount ();
+			response.Receipts = receipts;
 
 			++Count;
 
+			IndexHelperTool.ReportActivity ();
+
 			return response;
 		}
 	}
